/*
 * Created on 22 ott 2016
 * Copyright 2015 by Andrea Vacondio (andrea.vacondio@gmail.com).
 * This file is part of Sejda.
 *
 * Sejda is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Sejda is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Sejda.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.sejda.cli.transformer;

import org.sejda.cli.model.WatermarkTaskCliArguments;
import org.sejda.model.parameter.WatermarkParameters;

/**
 * {@link CommandCliArgumentsTransformer} for the Watermark task command line interface
 * 
 * @author Andrea Vacondio
 *
 */
public class WatermarkCliArgumentsTransformer extends BaseCliArgumentsTransformer
        implements CommandCliArgumentsTransformer<WatermarkTaskCliArguments, WatermarkParameters> {

    @Override
    public WatermarkParameters toTaskParameters(WatermarkTaskCliArguments args) {
        WatermarkParameters parameters = new WatermarkParameters(args.getWatermark().getSource());
        if (args.isDimension()) {
            parameters.setDimension(args.getDimension().getDimension());
        }
        parameters.setLocation(args.getLocation().getEnumValue());
        parameters.setOpacity(args.getAlpha());
        parameters.setPosition(args.getPosition().getPoint());

        if (args.isPageSelection()) {
            parameters.addAllPageRanges(args.getPageSelection().getPageRangeSet());
        }

        populateAbstractParameters(parameters, args);
        populateSourceParameters(parameters, args);
        populateOutputTaskParameters(parameters, args);
        populateOutputPrefix(parameters, args);
        return parameters;
    }
}
